# Make Regression Data: Two Class Bivariate CAR with Psi Class-Specific Phis.r
# NC Adj Matrix
# Simulated Two-Class Bivariate Data with covariate x and BiCAR random effects
# Infiles:  1) C:Brian/CARmix/R/Final Sim/Makedata/County A.txt (Adj Matrix)
#		        2) C:Brian/CARmix/R/Final Sim/Makedata/writeDatafileR.txt
# Outfile:  C:Brian/CARmix/R/Final Sim/Data/two class regression data.txt
# Oct 22, 2011
#########################
library(mvtnorm)
library(simex) 

A<-dget("C:/Brian/CARmix/R/Final Sim/Makedata/County A.txt") 	# NC County Adj Matrix
source("C:/Brian/CARmix/R/Final Sim/Makedata/writeDatafileR.txt")

set.seed(102211)
 n<-100			            # Number of block groups in Durham County
 nis<-rep(25,n) 		    # Number of individuals per block; here it's balanced  
 tmp<-rbinom(n,1,.005)	# Empty blocks
 #nis[tmp==1]<-0
 id<-rep(1:n,nis)		
 N<-length(id) 		      # Total number of subjects

##################
# Adjacency Info #
##################
 m<-apply(A,1,sum)	# No. neighbors
 adjid<-rep(1:n,m)	# Tells which elements of A belong to which block
 adj<-rep(0,sum(m))
 for (i in 1:n)adj[adjid==i]<-which(A[i,]==1)  
  
#####################
#  Phis for Class 1 #
#####################
 rho<-.999999			     	# Spatial dependence parameter ~ 1 for intrinsic CAR
 cov1<-matrix(c(4,6,6,16),2,2)   	# Conditional Cov of phi1 and phi2 given phi_{-i} 
 Q<-diag(m)-rho*A			     	# NOTE: Indt random effects ==>  rho=0 & m=1 for all i

 covphi1<-solve(Q)%x%cov1			# 2n x 2n covariance of phis
 phi1<-rmvnorm(1,sigma=covphi1)		# 2n vector of spatial effects
 phitmp1<-matrix(phi1,ncol=2,byrow=T) 	# n x 2 matrix of spatial effects
 Phi11<-rep(phitmp1[,1]-mean(phitmp1[,1]),nis) # N x 1 phi1 vector -- Centered
 Phi12<-rep(phitmp1[,2]-mean(phitmp1[,2]),nis) # N x 1 phi2 vector 
 true.phi1<-t(t(phitmp1)-apply(phitmp1,2,mean))# Phi1 and Phi2 for Class 1

###########
#  Phi2   #
###########
 cov2<-matrix(c(2,2,2,4),2,2)     # Conditional Cov of phi1 and phi2 given phi_{-i} 
 covphi2<-solve(Q)%x%cov2			    # 2n x 2n covariance of phis
 phi2<-rmvnorm(1,sigma=covphi2)		# 2n vector of spatial effects
 phitmp2<-matrix(phi2,ncol=2,byrow=T) 	# n x 2 matrix of spatial effects
 Phi21<-rep(phitmp2[,1]-mean(phitmp2[,1]),nis) # N x 1 phi1 vector -- Centered
 Phi22<-rep(phitmp2[,2]-mean(phitmp2[,2]),nis) # N x 1 phi2 vector 
 true.phi2<-t(t(phitmp2)-apply(phitmp2,2,mean))

##########
#  Psi   #
##########
s2psi<-1					                  # Var psi
covpsi<-solve(Q)*s2psi
psitmp<-c(rmvnorm(1,sigma=covpsi))  
psi<-true.psi<-psitmp-mean(psitmp)	# Center		
Psi<-rep(psi,nis)

##############################
# Fixed Effects and Response ############
#  JUST USE INTERCEPTS FOR THIS EXAMPLE #
#########################################
 K<-2 			# Number of classes
 # Class indicator  
    w<-rbinom(N,1,.5)
    W<-cbind(1,w)
    gamma<-c(.75,-.5)		
    etag<-W%*%gamma+Psi
    p<-exp(etag)/(1+exp(etag))
    C<-true.c<-rbinom(N,1,p)+1
    N1<-table(C)[1]; N2<-table(C)[2]

 # Covariate
   x<-sample(0:4,N,replace=T)
   X<-cbind(rep(1,N),x)		# N x p design matrix 
   Y<-matrix(0,N,2)			  # N x 2 response matrix
   Y1<-matrix(0,N1,2)
   Y2<-matrix(0,N2,2)

 ###############
 # Component 1 #
 ###############
  beta1<-c(.75,-.75)  # Fixed effects for y1	# Reg parameters y1 (Math), component 1			
  beta2<-c(-1,.5)		  # Intercept for y2 (Reading), component 1
  eta1<-X[C==1,]%*%beta1+Phi11[C==1]
  eta2<-X[C==1,]%*%beta2+Phi12[C==1]
  eta<-cbind(eta1,eta2)
  cov<-matrix(c(1,-1,-1,3),2,2) # Conditional Cov of y1,y2 given phis and x
  for (i in 1:N1) Y1[i,]<-rmvnorm(1,eta[i,],cov)	
  Y1[,1]<-Y1[,1]
  Y[C==1,]<-Y1

 ################
 # Component  2 #
 ################
  beta1<-c(-.75,.75)  # Fixed effects for y1  # Reg parameters y1 (Math), component 1			
  beta2<-c(1,-.5)		  # Intercept for y2 (Reading), component 1
  eta1<-X[C==2,]%*%beta1+Phi21[C==2]
  eta2<-X[C==2,]%*%beta2+Phi22[C==2]
  eta<-cbind(eta1,eta2)
  cov<-matrix(c(2,1,1,2),2,2) # Conditional Cov of y1,y2 given phis and x
  for (i in 1:N2) Y2[i,]<-rmvnorm(1,eta[i,],cov)	
  Y[C==2,]<-Y2
  y1<-Y[,1]
  y2<-Y[,2]

 data<-list(n=n,nis=nis,y1=y1,y2=y2,x=x,w=w,id=id)
 writeDatafileR(data,"C:/Brian/CARmix/R/Data/two class regression data.txt")
