# Two Class Regression MATNORM.r
# Runs MCMC for fitting Two-Component Finite Mixture Linear Regression Model with covariate x and BiCAR random effects
# ADDS CONJUGATE MATNORM-IW JOINT PRIOR FOR Beta|Sigmae and Sigmae within each mixture component
#   GIVES ESSENTIALLY SAME RESULTS AS PREVIOUS PROGRAM, SINCE WE ASSUME WEAK PRIORS
# CONDITONAL UPDATE FOR PHI1|PHI2 and PHi2|Phi1
# NOTE: Can write conditional prior of phi1|phi2 as: 
#       phi1|phi2 ~ N(mu, Sigma) where Sigma = [sphi1^2(1-rho^2)] and mu = rho*sphi1/sphi2 *phi2 
# Infile:   1) Data = C:/Brian/CARmix/R/Final Sim/Data/two class regression data.txt" 
#		        2) A County.txt = NC County Adjacency Matrix 
#		        3) Update Gamma Psi.r, Update C Psi.r.
# June 2, 2014
##############################
library(mvtnorm)
library(MCMCpack) 	# iwish distribution used in updating Sigma
library(Hmisc)		  # For updating C
library(spam)

A<-dget("F:/CARmix/R/Final Sim/Makedata/County A.txt") 		    # NC Adj Matrix
source ("F:/CARmix/R/Final Sim/Functions/Update Gamma.r")     # Update Gamma coefficients (mixture compnent reg. parms)
source ("F:/CARmix/R/Final Sim/Functions/Update C.r")         # Update component indicators, C

########
# Data #
########
 data<-dget("F:/CARmix/R/Final Sim/Data/two class regression data.txt")
 K<-2			# Number of classes
 h<-2     # Number of outcomes
 y1<-data$y1
 y2<-data$y2
 x<-data$x
 w<-data$w
 Y<-cbind(y1,y2)
 id<-data$id
 N<-length(y1)
 nis<-data$nis
 n<-data$n
 X<-cbind(1,x)
 p<-ncol(X)
 W<-cbind(1,w)
 r<-ncol(W)

##################
# Adjacency Info #
##################
 nneighb<-apply(A,1,sum)	# No. neighbors
 adjid<-rep(1:n,nneighb)	# Tells which elements of A belong to which block -- NEED FOR UPDATING PSI
 adj<-rep(0,sum(nneighb))
 for (i in 1:n)adj[adjid==i]<-which(A[i,]==1) 

#############
# MCMC Info #
#############
 nsim<-1000				# Number of MCMC Iterations
 thin<-1          # Thinning Interval (should be a divisor of nsim/2)
 burn<-nsim/2     # Burn-in
 lastit<-(nsim-burn)/thin	# Last stored value

##########
# Priors #
##########
  beta0<-matrix(0,p,h)  # Prior mean of (beta11 beta21
                        #                beta12 beta22)
  T0<-diag(.01,p)       # p x p Prior Precision beta1 and beta2, where p denotes number of parameters
  nu0<-3		            # DF for I-W prior on Sigma.phi
  nu<-nu0+n-1           # Posterior DF for Sigma.phi (Note1: each class gets n random effects; Note2: n-1 b/c rank of Q is n-1) 
  c0<-diag(0.01,2)	    # Scale matrix for I-W Prior on Sigma.phi

#########
# Inits #
#########
  set.seed(1001)
  beta1<-beta2<-matrix(0,K,p)   # beta1 and beta2 for each class
  betastar<-matrix(0,p,h)       # p x h matrix of beta1 (col 1), beta2 (col 2)
  betatilde2<-rep(0,K)          # betatilde2 = rho*sigmae_2/sigmae_1 for update C function NOTE:  CHANGED BETASTAR1 AND BETASTAR2 IN PREVIOUS PROGRAM
                                #                                                                 TO BETATILDE1 AND BETATILDE2 
  sigma1<-sigma21 <-rep(1,K)    # var(y1) and var(y2|y1) for each class -- need to store for update c function
  Sigmae<-diag(1,2*K)  	        # Cov  Y1,Y2 given phi;
  Sigmaphi<-diag(1,2*K)	        # Spatial covariance and precision
  c<-rbinom(N,1,0.50)+1		      # Component indicators
  gamma<-rep(0,r)               # Component fixed effect
  Q<-as.spam(diag(nneighb)-A)		# ICAR Q Matrix  
  phi1<-phi2<-matrix(0,K,n)   	# n x 1 vector of phi1 and ph2 for each class
  Phi1<-Phi2<-matrix(0,K,N)  		# N x 1 Phi1 and Phi2 for each class
  phimat<-array(0,dim=c(K,n,2)) # k x n x 2 matrix of phis for each class
  psi<-rep(0,n)			            # vector of psis
  Psi<-rep(0,N)  		 		        # N x 1 Psis
  sphi1<-sphi2<-rep(1,K)      	# prior SDs for phi1 and phi2 for each class     
  rho.phi<-rep(0,K)           	# Corr(phi1_k,phi2_k) for k=1,2
  phitmp2<-nistmp<-rep(0,n)     # Vector for updating phi's when some component-specific nis = 0

######################
# Vectors needed for # 
# MH update of psi   #
######################
  lsum_psi<-rep(0,n)
  psibar<-rep(0,n)
  sigma2_psi<-1

#########
# Store #
#########
  Beta1<-Beta2<-matrix(0,lastit,K*p)	# Store Beta1 and Beta2 for each class
  Sige<-Sigphi<-matrix(0,lastit,4*K)
  Psis<-Phi11<-Phi12<-Phi21<-Phi22<-matrix(0,lastit,n)  	
  Sig2psi<-rep(0,lastit)
  Gamma<-matrix(0,lastit,r)
  Gtmp<-matrix(0,nsim,(K-1)*r)
  Cs<-matrix(0,lastit,N)

#########
# Gibbs #
#########
 ptm<-proc.time()

 for (i in 1:nsim) {

 #######################
 # Update latent class #
 #  Probabilities	     #
 #######################  
    gamma<-update.gamma(g=gamma,W=W,c=c,s=100,Psi=Psi,tau=.25,burn=nsim/4,it=i,G=Gtmp)  # tau is tuning parameter -- lower this if gamma won't update

 ##############################
 # Update class indicators, c #
 ##############################
    c<-update.c(g=gamma,y1=y1,y2=y2,X=X,W=W,Phi1=Phi1,Phi2=Phi2,Psi=Psi,
    		beta1=beta1,beta2=beta2,sigma1=sigma1,sigma21=sigma21,betastar2=betatilde2)

    ns<-table(c)
 
 ########################################
 # Update Component-Specific Parameters #
 ########################################
    for (k in 1:K) {
      
      # Update beta (update V and m using Matrix Normal form, then vectorize to form mstar)
        V<-solve(T0+crossprod(X[c==k,]))                                                    # p x p post. covariance 
        m<-V%*%(T0%*%beta0+t(X[c==k,])%*%(Y[c==k,]-t(rbind(Phi1[k,c==k],Phi2[k,c==k]))))    # p x h, where p is number of covariates (including int.) and h is number of outcomes
        mstar<-c(m)                                                                         # p*h x 1 = (beta11, beta12, beta21, beta22)
        beta<-c(rmvnorm(1,mstar,Sigmae[(2*(k-1)+1):(2*k),(2*(k-1)+1):(2*k)]%x%V))           # p*h x 1 row-vector
        betastar<-matrix(beta,p,h)            # p x h matrix
        beta1[k,]<-beta[1:p]                  # Coeff for y1 for class k
        beta2[k,]<-beta[(p+1):(p*h)]          # Coeff for y2 for class k
      
      # Update Sigmae_k and error variance components
        phitmp<-c(t(cbind(Phi1[k,c==k],Phi2[k,c==k])))
        ytmp<-c(t(Y[c==k,]))
        Xtmp1<-kronecker(X[c==k,],c(1,0))
        Xtmp2<-kronecker(X[c==k,],c(0,1))
        Xsigma<-cbind(Xtmp1,Xtmp2)
        resid<-matrix(ytmp-Xsigma%*%beta-phitmp,ns[k],2,byrow=T)
        Sigmae[(2*(k-1)+1):(2*k),(2*(k-1)+1):(2*k)]<-riwish(ns[k]+nu0+p,c0+crossprod(resid)+t(betastar-beta0)%*%T0%*%(betastar-beta0))   # Var-Cov [(Y1,Y2)|Phi]
      
        sigma1[k]<-Sigmae[(2*(k-1)+1),(2*(k-1)+1)]  # Var(Y1|Phi) -- Store for udpate c function
        sigma2<-Sigmae[(2*(k-1)+2),(2*(k-1)+2)]     # Var(Y2|phi)
        rho<-Sigmae[(2*(k-1)+1),(2*(k-1)+2)]/sqrt(sigma1[k]*sigma2)
        tau12<-1/((1-rho^2)*sigma1[k])              # Posterior Precision y1|y2
        sigma21[k]<-(1-rho^2)*sigma2                # Posterior var(y2|y1) -- Need to store of update c
        tau21<-1/sigma21[k]
        betatilde1<-rho*sqrt(sigma1[k]/sigma2) 
        betatilde2[k]<-rho*sqrt(sigma2/sigma1[k])   # Need to store for update c

      # Update Sigmaphi and variance components
        Sigmaphi[(2*(k-1)+1):(2*k),(2*(k-1)+1):(2*k)]<-tmp<-riwish(nu,c0+t(phimat[k,,])%*%Q%*%phimat[k,,])	
        sphi1<-sqrt(tmp[1,1])               # Sd phi1
        sphi2<-sqrt(tmp[2,2])               # Sd phi2
        rho.phi<-tmp[1,2]/(sphi1*sphi2)     # Corr(phi1,phi2)
        
      # Update phi1_k|phi2_k 
        tmp1<-y2[c==k]-X[c==k,]%*%beta2[k,]-Phi2[k,c==k]   	  
	      tmp<-table(id[c==k])
    	  nistmp[as.numeric(names(tmp))]<-tmp   	  	# nis for class k (allow for empty spatial units)
        priorprec<-1/(sphi1^2*(1-rho.phi^2))*Q    	# Prior precision of phi1|phi2
        priormean<-rho.phi*sphi1/sphi2*phi2[k,]   	# Prior mean of phi1|phi2 
        precphi1<-priorprec+tau12*diag(nistmp)    	# Posterior precision
	      tmp<-tau12*tapply(y1[c==k]-X[c==k,]%*%beta1[k,]-betatilde1*tmp1,id[c==k],sum)    
	      phitmp2[as.numeric(names(tmp))]<-tmp	  	  # Account for empty spatial units
    	  mphi1<-c(priorprec%*%priormean) + phitmp2
        phi1[k,]<-rmvnorm.canonical(1, mphi1, precphi1)[1,]
        
      # Update phi2_k|phi1_k
        tmp2<-y1[c==k]-X[c==k,]%*%beta1[k,]-Phi1[k,c==k]
        priorprec<-1/(sphi2^2*(1-rho.phi^2))*Q    	# Prior precision of phi1|phi2
        priormean<-rho.phi*sphi2/sphi1*phi1[k,]   	# Prior mean of phi1|phi2 
        precphi2<-priorprec+tau21*diag(nistmp)      	# Posterior precision
	      tmp<-tau21*tapply(y2[c==k]-X[c==k,]%*%beta2[k,]-betatilde2[k]*tmp2,id[c==k],sum)    
	      phitmp2[as.numeric(names(tmp))]<-tmp	    	# Account for empty units
    	  mphi2<-c(priorprec%*%priormean) + phitmp2
        phi2[k,]<-rmvnorm.canonical(1, mphi2, precphi2)[1,]

      # Center Phis
    	  phi1[k,]<-phi1[k,]-mean(phi1[k,])  		# n x 1 phi1 vector for each class
    	  phi2[k,]<-phi2[k,]-mean(phi2[k,])  		# n x 1 phi2 vector for each class
    	  phimat[k,,]<-cbind(phi1[k,],phi2[k,])	
    	  Phi1[k,]<-rep(phi1[k,],nis)				# N x 1 phi1 vector for each class
    	  Phi2[k,]<-rep(phi2[k,],nis)				# N x 1 phi1 vector for each class
 
  } # End k-loop

 #######################
 # Update Psi using MH #
 #######################
  psinew<-psi+rt(n,3)
  Psinew<-rep(psinew,nis)
  etapsi<-W%*%t(gamma)+Psi		# Old linear predictor
  etapsinew<-W%*%t(gamma)+Psinew	# New LP
  ppsi<-exp(etapsi)/(1+exp(etapsi))
  ppsinew<-exp(etapsinew)/(1+exp(etapsinew))

  lsum_psi[nis!=0]<-tapply(dbinom(c-1,1,ppsinew,log=T)-dbinom(c-1,1,ppsi,log=T),id,sum)
  
  for (j in 1:n){
     psibar[j]<-mean(psi[adj[which(adjid==j)]])
     rpsi<- lsum_psi[j] + dnorm(psinew[j],psibar[j],sqrt(sigma2_psi/nneighb[j]),log=T)-
		                      dnorm(psi[j],psibar[j],sqrt(sigma2_psi/nneighb[j]),log=T)
     if (log(runif(1))<rpsi) psi[j]<-psinew[j] 
     
   }
   psi<-psi-mean(psi)
   Psi<-rep(psi,nis)
  
 ##################
 # Update tau_psi #
 ##################
   tau_psi<-rgamma(1,.01+(n-1)/2,.01+t(psi)%*%Q%*%psi/2)
   sigma2_psi<-1/tau_psi

 #################
 # Store Results #
 #################
   if (i> burn & i%%thin==0) {
    j<-(i-burn)/thin
    Beta1[j,]<-c(t(beta1))  # Store class 1 fixed effects
    Beta2[j,]<-c(t(beta2))  # Store class 2 fixed effects
    Gamma[j,]<-gamma        # Store gamma values
    Sige[j,]<-c(c(Sigmae[1:2,1:2],c(Sigmae[3:4,3:4])))        # Store Sigma.e values for each component
    Sigphi[j,]<-c(c(Sigmaphi[1:2,1:2],c(Sigmaphi[3:4,3:4]))) 	# Store Sigma.phi values for each component
    Sig2psi[j]<-sigma2_psi  # Store variance of psi
  }
  Gtmp[i,]<-gamma		        # Store at every it. for updt. gamma (Improves post-burnin mixing)
  if (i%%10==0) print(i)
}

tottime<-proc.time()-ptm

###########
# Results #
###########         # Note that class order could be switched
colMeans(Beta1)   	# Y1 Parameters for each class: True Value = c(0.75, -0.75, -0.75, 0.75) # Note class order could be switched
colMeans(Beta2)   	# Y2 Parameters for each class: True Value = c(-1, 0.50, 1, -0.50)
colMeans(Gamma)   	# True value = c(.75,-.5)
colMeans(Sige)	    # True Values = 1,-1,-1,3 and 2, 1, 1, 2
mean(Sig2psi)       # True value = 1
colMeans(Sigphi)    # True Values = 4, 6, 6, 16 and 2 2 2 4
t(apply(Sigphi,2,quantile,prob=c(0.025,0.975)))  # 95% CrI for Sigphi

###############
# Trace Plots #
###############
par(mfrow=c(3,2))
plot(1:lastit,Beta1[,1],type="l",col="lightgreen")
abline(h=mean(Beta1[,1]),col="blue")

plot(1:lastit,Gamma[,1],type="l",col="lightgreen")
abline(h=mean(Gamma[,1]),col="blue")

plot(1:lastit,Sige[,2],type="l",col="lightgreen")
abline(h=mean(Sige[,2]),col="blue")

plot(1:lastit,Sigphi[,3],type="l",col="lightgreen")
abline(h=mean(Sigphi[,3]),col="blue")

plot(1:lastit,Sig2psi,type="l",col="lightgreen")
abline(h=mean(Sig2psi),col="blue")