# Two Class Regression.r
# Runs MCMC for fitting Two-Component Finite Mixture Linear Regression Model with covariate x and BiCAR random effects
# JOINT UPDATE FOR PHI1,PHI2
# Infile:   1) Data = C:/Brian/CARmix/R/Final Sim/Data/two class regression data.txt" 
#		        2) A County.txt = NC County Adjacency Matrix 
#		        3) Update Gamma Psi.r, Update C Psi.r, etc.
# Oct 11, 2011
##############################
library(mvtnorm)
library(simex) 	  	# For Block Diag
library(MCMCpack) 	# iwish distribution used in updating Sigma
library(Hmisc)		  # For updating C

A<-dget("F:/CARmix/R/Final Sim/Makedata/County A.txt") 		    # NC Adj Matrix
source ("F:/CARmix/R/Final Sim/Functions/Update Gamma.r")     # Update Gamma coefficients (mixture compnent reg. parms)
source ("F:/CARmix/R/Final Sim/Functions/Update C.r")         # Update component indicators, C
source ("F:/CARmix/R/Final Sim/Functions/update phi.r")		    # Update bivariate phi function		
source ("F:/CARmix/R/Final Sim/Functions/update phi zero.r")	# Update bivariate phi function	for "empty" counties	

########
# Data #
########
 data<-dget("F:/CARmix/R/Final Sim/Data/two class regression data.txt")
 K<-2			# Number of classes
 y1<-data$y1
 y2<-data$y2
 x<-data$x
 w<-data$w
 Y<-cbind(y1,y2)
 id<-data$id
 N<-length(y1)
 nis<-data$nis
 n<-data$n
 X<-cbind(1,x)
 p<-ncol(X)
 W<-cbind(1,w)
 r<-ncol(W)

# Design matrix for updating sigmaE
  Xtmp1<-kronecker(X,c(1,0))
  Xtmp2<-kronecker(X,c(0,1))
  Xsigma<-cbind(Xtmp1,Xtmp2)

##################
# Adjacency Info #
##################
 nneighb<-apply(A,1,sum)	# No. neighbors
 adjid<-rep(1:n,nneighb)	# Tells which elements of A belong to which block
 adj<-rep(0,sum(nneighb))
 for (i in 1:n)adj[adjid==i]<-which(A[i,]==1) 

#############
# MCMC Info #
#############
 nsim<-1000				# Number of MCMC Iterations
 thin<-1          # Thinning Interval (should be a divisor of nsim/2)
 burn<-nsim/2     # Burn-in
 lastit<-(nsim-burn)/thin	# Last stored value

##########
# Priors #
##########
  a<-.1;b<-.1	  	    # Inv-Gamma hyperparms for updating tau2 (precision y2 | y1)
  beta0<-c(0,0)  	    # Prior mean of beta1 and beta2 for each class
  T0<-diag(.001,2)    # Prior Prec of beta1 and beta2 for each class
  nu0<-3		          # DF for I-W prior on Sigma.phi
  nu<-nu0+n-1	        # Posterior Shape parmaeter for IW --> 1 for island
  c0<-diag(2)	        # Scale matrix for I-W Prior on Sigma.phi

#########
# Inits #
#########
  set.seed(1001)
  beta1<-beta2<-matrix(0,K,p)		# beta1 and beta2 for each class
  betastar2<-rep(0,K)           # betastar2 for update C function
  sigma1<-sigma21 <-rep(1,K)    # var(y1) and var(y2|y1) for each class -- for update c function
  Sigmae<-Taue<-diag(1,2*K)  	  # Cov/Precision Y1,Y2 given phi;
  Sigmaphi<-Tauphi<-diag(1,2*K)	# Spatial covariance and precision
  c<-rbinom(N,1,0.50)+1  	      # Component indicators
  gamma<-c(0,0)
  Q<-diag(nneighb)-A		        # ICAR Q Matrix  
  phimat<-array(0,dim=c(K,n,2)) # k x n x 2 random effects matrix
  Phi1<-Phi2<-matrix(0,K,N)  	  # N x 1 Phi1 and Phi2 for each class
  phibar<-matrix(0,n,2) 	      # Mean spatial effects of neighbors for updating phi
  phi1c<-phi2c<-matrix(0,K,n)	  # Centered phi1, phi2 for each class
  psi<-rep(0,n)			            # vector of psis
  Psi<-rep(0,N)  		            # N x 1 Psis
  nik<-rep(0,n)			            # current nis per spatial unit for each class (used to update phi's)

######################
# Vectors needed for # 
# MH update of psi   #
######################
  lsum_psi<-rep(0,n)
  psibar<-rep(0,n)
  sigma2_psi<-1

#########
# Store #
#########
  Beta1<-Beta2<-matrix(0,lastit,K*p)	# Store Beta1 and Beta2 for each class
  Sige<-Sigphi<-matrix(0,lastit,4*K)
  Psis<-Phi11<-Phi12<-Phi21<-Phi22<-matrix(0,lastit,n)  	
  Sig2psi<-rep(0,lastit)
  Gamma<-matrix(0,lastit,r)
  Gtmp<-matrix(0,nsim,(K-1)*r)
  Cs<-matrix(0,lastit,N)

#########
# Gibbs #
#########
 ptm<-proc.time()

 for (i in 1:nsim) {

 #######################
 # Update latent class #
 #  Probabilities	     #
 #######################  
    gamma<-update.gamma(g=gamma,W=W,c=c,s=100,Psi=Psi,tau=.25,burn=nsim/4,it=i,G=Gtmp)  # tau is tuning parameter -- lower this if gamma won't update

 ##############################
 # Update class indicators, c #
 ##############################
    c<-update.c(g=gamma,y1=y1,y2=y2,W=W,Phi1=Phi1,Phi2=Phi2,Psi=Psi,
    		beta1=beta1,beta2=beta2,sigma1=sigma1,sigma21=sigma21,betastar2=betastar2)

    ns<-table(c)
 
 ########################################
 # Update Component-Specific Parameters #
 ########################################
    for (k in 1:K) {

      # Update Sigmae_k and other variance components
        phitmp<-c(t(cbind(Phi1[k,c==k],Phi2[k,c==k])))
        ytmp<-c(t(Y[c==k,]))
        Xtmp1<-kronecker(X[c==k,],c(1,0))
        Xtmp2<-kronecker(X[c==k,],c(0,1))
        Xsigma<-cbind(Xtmp1,Xtmp2)
        resid<-matrix(ytmp-Xsigma%*%c(beta1[k,],beta2[k,])-phitmp,ns[k],2,byrow=T)
        Sigmae[(2*(k-1)+1):(2*k),(2*(k-1)+1):(2*k)]<-riwish(ns[k]+nu0,c0+crossprod(resid))   # Var-Cov [(Y1,Y2)|Phi]
      
        sigma1[k]<-Sigmae[(2*(k-1)+1),(2*(k-1)+1)]  # Store for udpate c
        sigma2<-Sigmae[(2*(k-1)+2),(2*(k-1)+2)]
        rho<-Sigmae[(2*(k-1)+1),(2*(k-1)+2)]/sqrt(sigma1[k]*sigma2)
        tau12<-1/((1-rho^2)*sigma1[k])        # Posterior Precision y1|y2
        sigma21[k]<-(1-rho^2)*sigma2          # Posterior var(y2|y1) -- Need to store of update c
        tau21<-1/sigma21[k]
        betastar1<-rho*sqrt(sigma1[k]/sigma2) 
        betastar2[k]<-rho*sqrt(sigma2/sigma1[k]) # Need to store for update c
		    Taue[(2*(k-1)+1):(2*k),(2*(k-1)+1):(2*k)]<-solve(Sigmae[(2*(k-1)+1):(2*k),(2*(k-1)+1):(2*k)])	# Cond'l precision of Sigmae for class k, given Phi
   
      # Update beta1 
        tmp<-y2[c==k]-X[c==k,]%*%beta2[k,]-Phi2[k,c==k]
        v<-solve(T0+tau12*crossprod(X[c==k,]))  
        m<-v%*%(T0%*%beta0+tau12*t(X[c==k,])%*%(y1[c==k]-betastar1*tmp-Phi1[k,c==k]))  
        beta1[k,]<-rmvnorm(1,m,v)

      # Update beta2
        tmp<-y1[c==k]-X[c==k,]%*%beta1[k,]-Phi1[k,c==k]
        v<-solve(T0+tau21*crossprod(X[c==k,]))  
        m<-v%*%(T0%*%beta0+tau21*t(X[c==k,])%*%(y2[c==k]-betastar2[k]*tmp-Phi2[k,c==k]))  
        beta2[k,]<-rmvnorm(1,m,v)

	    # Update Phis
   	   for (j in 1:n){
  		  nik[j]<-ifelse(nis[j]>0,length(id[c==k&id==j]),0)
		    phibar[j,]<-apply(phimat[k,adj[which(adjid==j)],],2,mean) 

		    if (nik[j]>0) phimat[k,j,]<-update.phi( y1=y1[c==k&id==j],y2=y2[c==k&id==j],
				  	                                    X=X[c==k&id==j,],phibar=phibar[j,],ni=nik[j],mi=nneighb[j],
				 	                                      beta=c(beta1[k,],beta2[k,]),Tau=Taue[(2*(k-1)+1):(2*k),(2*(k-1)+1):(2*k)],
				  	                                    Tauphi=Tauphi[(2*(k-1)+1):(2*k),(2*(k-1)+1):(2*k)])

   		  if (nik[j]==0) phimat[k,j,]<-update.phizero(phibar=phibar[j,],mi=nneighb[j],
				  		                                      Tau=Taue[(2*(k-1)+1):(2*k),(2*(k-1)+1):(2*k)],
				  		                                      Tauphi=Tauphi[(2*(k-1)+1):(2*k),(2*(k-1)+1):(2*k)])
 
	    } # End j-loop
    
    	phi1c[k,]<-phimat[k,,1]-mean(phimat[k,,1])  # n x 1 phi1 vector for each class
    	phi2c[k,]<-phimat[k,,2]-mean(phimat[k,,2])  # n x 1 phi2 vector for each class
    	phimat[k,,]<-cbind(phi1c[k,],phi2c[k,])	
    	Phi1[k,]<-rep(phi1c[k,],nis)			# N x 1 phi1 vector for each class
    	Phi2[k,]<-rep(phi2c[k,],nis)			# N x 1 phi1 vector for each class
 
    # Update Sigmaphi and Tauphi
  	  Sigmaphi[(2*(k-1)+1):(2*k),(2*(k-1)+1):(2*k)]<-riwish(nu,c0+t(phimat[k,,])%*%Q%*%phimat[k,,])	
	    Tauphi[(2*(k-1)+1):(2*k),(2*(k-1)+1):(2*k)]<-solve(Sigmaphi[(2*(k-1)+1):(2*k),(2*(k-1)+1):(2*k)]) 

  } # End k-loop

 #######################
 # Update Psi using MH #
 #######################
  psinew<-psi+rt(n,3)
  Psinew<-rep(psinew,nis)
  etapsi<-W%*%t(gamma)+Psi		# Old linear predictor
  etapsinew<-W%*%t(gamma)+Psinew	# New LP
  ppsi<-exp(etapsi)/(1+exp(etapsi))
  ppsinew<-exp(etapsinew)/(1+exp(etapsinew))

  lsum_psi[nis!=0]<-tapply(dbinom(c-1,1,ppsinew,log=T)-dbinom(c-1,1,ppsi,log=T),id,sum)
  
  for (j in 1:n){
     psibar[j]<-mean(psi[adj[which(adjid==j)]])
     rpsi<- lsum_psi[j] + dnorm(psinew[j],psibar[j],sqrt(sigma2_psi/nneighb[j]),log=T)-
		dnorm(psi[j],psibar[j],sqrt(sigma2_psi/nneighb[j]),log=T)
     if (log(runif(1))<rpsi) psi[j]<-psinew[j] 
     
   }
   psi<-psi-mean(psi)
   Psi<-rep(psi,nis)
  
 ##################
 # Update tau_psi #
 ##################
   tau_psi<-rgamma(1,.01+(n-1)/2,.01+t(psi)%*%Q%*%psi/2)
   sigma2_psi<-1/tau_psi

 #################
 # Store Results #
 #################
   if (i> burn & i%%thin==0) {
    j<-(i-burn)/thin
    Beta1[j,]<-c(t(beta1))  # Store Beta1 for each component
    Beta2[j,]<-c(t(beta2))  # Store Beta2 for each component
    Gamma[j,]<-gamma        # Store gamma values
    Sige[j,]<-c(c(Sigmae[1:2,1:2],c(Sigmae[3:4,3:4])))        # Store Sigma.e values for each component
    Sigphi[j,]<-c(c(Sigmaphi[1:2,1:2],c(Sigmaphi[3:4,3:4]))) 	# Store Sigma.phi values for each component
    Sig2psi[j]<-sigma2_psi  # Store variance of psi
    #Psis[j,]<-psi			      # Store psi values
    #Phi11[j,]<-phi1c[1,]		# phi1 for class 1
    #Phi12[j,]<-phi2c[1,]		# phi2 for class 1
    #Phi21[j,]<-phi1c[2,]		# phi1 for class 2
    #Phi22[j,]<-phi2c[2,]		# phi2 for class 2
    #Cs[j,]<-c
  }

  Gtmp[i,]<-gamma		        # Store at every it. for updt. gamma (Improves post-burnin mixing)
  if (i%%10==0) print(i)
}

tottime<-proc.time()-ptm

###########
# Results #
###########
colMeans(Beta1)   	# True Value = c(0.75, -0.75, -0.75, 0.75)
colMeans(Beta2)   	# True Value = c(-1, 0.50, 1, -0.50)
colMeans(Gamma)   	# True value = c(.75,-.5)
colMeans(Sige)	    # True Values = 1,-1,-1,3 and 2, 1, 1, 2
mean(Sig2psi)       # True value = 1
colMeans(Sigphi)    # True Values = 4, 6, 6, 16 and 2 2 2 4
t(apply(Sigphi,2,quantile,prob=c(0.025,0.975)))  # 95% CrI for Sigphi

###############
# Trace Plots #
###############
par(mfrow=c(3,2))
plot(1:lastit,Beta1[,1],type="l",col="lightgreen")
abline(h=mean(Beta1[,1]),col="blue")

plot(1:lastit,Gamma[,1],type="l",col="lightgreen")
abline(h=mean(Gamma[,1]),col="blue")

plot(1:lastit,Sige[,2],type="l",col="lightgreen")
abline(h=mean(Sige[,2]),col="blue")

plot(1:lastit,Sigphi[,3],type="l",col="lightgreen")
abline(h=mean(Sigphi[,3]),col="blue")

plot(1:lastit,Sig2psi,type="l",col="lightgreen")
abline(h=mean(Sig2psi),col="blue")