# Update C (Class Indicators and Corresponding Sample Sizes)
# Adds class-membership random effect, Psi
# ADDS CLASS SPECIFIC PHIS
# CONDITIONAL BIVARIATE NORMAL f(Y1)xf(Y2|y1) -- SEE: Scott and Alan's SIM paper
# Function Parameters:
  # gamma = 2 x r matrix of class-membership reg. parameters (K and r >= 2)
  #  		 note 1: last gamma=rep(0,r) for identification purposes
  #		 note 2: if r=1 can use Dirichlet prior of class indicators
  # y1 and y2 =  n x 1 stacked response vectors 
  # W = n x r matrix of class membership covariates including intercept
  # X = n x p matrix of covariates (including intercept)
  # beta1 = K x p matrix of means for Y1 (rows correspond to classes)
  # beta2 = K x p matrix of means for Y2
  # sigma1 = K x 1 vector of variances for y1
  # sigma21 = K x 1 variances for y2|y1
  # betastar2= K x 1 vector of rho*sigma2/sigma1 values for each class
  # Phi1,Phi2 = K x N matrices of random effects on mean of y1,y2 for each class
  # psi = n x 1 vector of class-membership spatial random effects

# NOTE: MUST LOAD Hmisc Library for rMultinom function
# August 11, 2011
#######################

update.c<-function(g=gamma,y1=y1,y2=y2,X=X,W=W,Phi1=Phi1,Phi2=Phi2,
		Psi=Psi,beta1=beta1,beta2=beta2,sigma1=sigma1,sigma21=sigma21,betastar2=betastar2) {

 W<-as.matrix(W)
 n<-nrow(W)				# Number of subjects
 r<-ncol(W)				# Number of class reg parms, incl intcpt			

 if (is.vector(g)==TRUE) g<-matrix(g,1,ncol=r)	# Convert g to matrix if vector
 K<-nrow(g)+1  			# Number of classes
 
 eta<-cbind(rep(0,n),W%*%t(g)+Psi)
 pi<-exp(eta)/(1+apply(as.matrix(exp(eta[,-1])),1,sum)) # Class proportions, given W
 
 # Likelihood contributions, Classes 1-K
   l<-matrix(0,n,K)		# Like contribution for subject i, class k
   pnum<-matrix(0,n,K)		# Numerator for post prob of C_i=k
 
 for (k in 1:K){
   mutmp1<-X%*%beta1[k,]+Phi1[k,]
   mutmp2<-X%*%beta2[k,]+Phi2[k,]+betastar2[k]*(y1-mutmp1)
   l[,k]<-dnorm(y1,mutmp1,sqrt(sigma1[k]))*dnorm(y2,mutmp2,sqrt(sigma21[k]))
   pnum[,k]<-pi[,k]*l[,k]
 }

 p<-pnum/(apply(pnum,1,sum))
 p[is.na(p)==1]<-rep(1/K,K)
 c<-rMultinom(p,1)

 return(c)
}




