# Update phi (spatial effects) for Bicar model
# Assumes Improper BiCAR model:
#	 phi_i|phi_(-i),Sigma.phi ~ N(1/m_i sum_{l in Q}\phi_l, Sigma.phi/mi)
# Inputs:
#	ni 		= number of observations in block i (ASSUME > 0)
#     mi 		= # neighbors for block i
#     y1		= n_i x 1 y1 vector
#     y2		= n_i x 1 y2 vector
#	X		= n_i x p covriate matrix (ASSUMED SAME FOR BOTH y1 AND y2)
#	phibar	= mean spatial effect for block neighbors
# 	beta		= 2p x 1 fixed effect regression coefficients
#	Tau		= 2x2 Within-Block Error Precision
#	Tauphi	= 2x2 Spatial Precision
# Returns: 2 x 1 phi_i vector
# March 15, 2011
################################################
update.phi<-function(y1=y1[id==j],y2=y2[id==j],X=X[id==j,],phibar=phibar[j,],
	     ni=nis[j],mi=m[j],beta=beta,Tau=Tau,Tauphi=Tauphi){
  y<-c(t(cbind(y1,y2)))
  if(is.vector(X)) X<-t(X)
  X1<-kronecker(X,c(1,0))
  X2<-kronecker(X,c(0,1)) 
  X<-cbind(X1,X2)		# 2ni x 2p design matrix 
  Z<-cbind(rep(1,ni)%x%c(1,0),rep(1,ni)%x%c(0,1))  # CAN CREATE BIG Z OUTSIDE
  V<-solve(ni*Tau+mi*Tauphi)
  m<-V%*%(Tau%*%crossprod(Z,y-X%*%beta)+mi*Tauphi%*%phibar)
  phi<-rmvnorm(1,m,V)
 return(phi)
}